@@ -30,9 +30,10 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
+from typing import Optional
 from gettext import gettext as _
 
-from gi.repository import Gtk, Gio, GLib, Gdk, GObject
+from gi.repository import Adw, Gtk, Gio, GLib, Gdk, GObject
 
 from gnomemusic.coregrilo import CoreGrilo
 from gnomemusic.coremodel import CoreModel
@@ -50,7 +51,7 @@ from gnomemusic.widgets.lastfmdialog import LastfmDialog
 from gnomemusic.window import Window
 
 
-class Application(Gtk.Application):
+class Application(Adw.Application):
 
     def __init__(self, application_id):
         super().__init__(
@@ -61,7 +62,6 @@ class Application(Gtk.Application):
         GLib.set_prgname(application_id)
         GLib.setenv("PULSE_PROP_media.role", "music", True)
 
-        self._init_style()
         self._window = None
 
         self._log = MusicLogger()
@@ -78,17 +78,11 @@ class Application(Gtk.Application):
         self._lastfm_scrobbler = LastFmScrobbler(self)
         self._player = Player(self)
 
+        self._lastfm_dialog: Optional[LastfmDialog] = None
+
         InhibitSuspend(self)
         PauseOnSuspend(self._player)
 
-    def _init_style(self):
-        css_provider = Gtk.CssProvider()
-        css_provider.load_from_resource('/org/gnome/Music/org.gnome.Music.css')
-        screen = Gdk.Screen.get_default()
-        style_context = Gtk.StyleContext()
-        style_context.add_provider_for_screen(
-            screen, css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
-
     @GObject.Property(
         type=CoreGrilo, default=None, flags=GObject.ParamFlags.READABLE)
     def coregrilo(self):
@@ -202,16 +196,33 @@ class Application(Gtk.Application):
             if accel is not None:
                 self.set_accels_for_action(*accel)
 
-    def _help(self, action, param):
-        try:
-            Gtk.show_uri(None, "help:gnome-music", Gdk.CURRENT_TIME)
-        except GLib.Error:
-            self._log.message("Help handler not available.")
+    def _help(self, action: Gio.Action, param: Optional[GLib.Variant]) -> None:
 
-    def _lastfm_account(self, action, param):
-        lastfm_dialog = LastfmDialog(self._window, self._lastfm_scrobbler)
-        lastfm_dialog.run()
-        lastfm_dialog.destroy()
+        def show_uri_cb(parent: Gtk.Window, result: Gio.AsyncResult) -> None:
+            try:
+                Gtk.show_uri_full_finish(parent, result)
+            except GLib.Error:
+                self._log.message("Help handler not available.")
+
+        Gtk.show_uri_full(
+            self._window, "help:gnome-music", Gdk.CURRENT_TIME, None,
+            show_uri_cb)
+
+    def _lastfm_account(
+            self, action: Gio.SimpleAction,
+            param: Optional[GLib.Variant]) -> None:
+
+        def on_response(dialog: LastfmDialog, response_id: int) -> None:
+            if not self._lastfm_dialog:
+                return
+
+            self._lastfm_dialog.destroy()
+            self._lastfm_dialog = None
+
+        self._lastfm_dialog = LastfmDialog(
+            self._window, self._lastfm_scrobbler)
+        self._lastfm_dialog.connect("response", on_response)
+        self._lastfm_dialog.present()
 
     def _about(self, action, param):
         about = AboutDialog()
@@ -219,7 +230,9 @@ class Application(Gtk.Application):
         about.present()
 
     def do_startup(self):
-        Gtk.Application.do_startup(self)
+        Adw.Application.do_startup(self)
+        Adw.StyleManager.get_default().set_color_scheme(
+            Adw.ColorScheme.PREFER_LIGHT)
         self._set_actions()
 
     def _quit(self, action=None, param=None):
