@@ -72,9 +72,15 @@ class AsyncQueue(GObject.GObject):
         """
         async_obj_id = id(args[0])
 
-        if (async_obj_id not in self._async_pool
-                and async_obj_id not in self._async_active_pool):
-            self._async_pool[async_obj_id] = (args)
+        if async_obj_id in self._async_active_pool:
+            obj = args[0]
+            handler_id, _ = self._async_data.pop(obj)
+            obj.disconnect(handler_id)
+            self._async_active_pool.pop(id(obj))
+        elif async_obj_id in self._async_pool:
+            self._async_pool.pop(async_obj_id)
+
+        self._async_pool[async_obj_id] = (args)
 
         if self._timeout_id == 0:
             self._timeout_id = GLib.timeout_add(100, self._dispatch)
@@ -97,6 +103,11 @@ class AsyncQueue(GObject.GObject):
                 tick)
             async_obj.start(*async_task_args[1:])
 
+        self._log.debug(
+            f"{self._queue_name}: "
+            f"{len(self._async_active_pool)} active task(s)"
+            f"with {len(self._async_pool)} remaining")
+
         return GLib.SOURCE_CONTINUE
 
     def _on_async_finished(
@@ -105,10 +116,5 @@ class AsyncQueue(GObject.GObject):
         t = (time.time() - tick) * 1000
         self._log.debug(f"{self._queue_name}: {t:.2f} ms task")
 
-        a = len(self._async_active_pool)
-        self._log.debug(
-            f"{self._queue_name}: "
-            f"{a} active task(s) of {len(self._async_pool) + a}")
-
         obj.disconnect(handler_id)
         self._async_active_pool.pop(id(obj))
