@@ -22,36 +22,38 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-import math
+from __future__ import annotations
+from typing import Any, Optional, Union
+import typing
 
-import gi
-gi.require_version("Gfm", "0.1")
-from gi.repository import GObject, Gio, Gfm, Gtk
+from gi.repository import GLib, GObject, Gio, Gtk
 
 from gnomemusic.corealbum import CoreAlbum
 from gnomemusic.coreartist import CoreArtist
 from gnomemusic.coresong import CoreSong
 from gnomemusic.grilowrappers.grltrackerplaylists import Playlist
 from gnomemusic.player import PlayerPlaylist
-from gnomemusic.songliststore import SongListStore
 from gnomemusic.widgets.songwidget import SongWidget
+if typing.TYPE_CHECKING:
+    from gnomemusic.application import Application
+    from gnomemusic.search import Search
 import gnomemusic.utils as utils
 
 
 class CoreModel(GObject.GObject):
-    """Provides all the list models used in Music
+    """Provides all the global list models used in Music
 
     Music is using a hierarchy of data objects with list models to
     contain the information about the users available music. This
     hierarchy is filled mainly through Grilo, with the exception of
     playlists which are a Tracker only feature.
 
-    There are three main models: one for artist info, one for albums
-    and one for songs. The data objects within these are CoreArtist,
-    CoreAlbum and CoreSong respectively.
+    There are three main models: one for artists, one for albums and
+    one for songs. The data objects within these are CoreArtist,
+    CoreAlbum and CoreSong respectively. These models are flattened
+    list models filled by the wrappers.
 
-    The data objects contain filtered lists of the three main models.
-    This makes the hierarchy as follows.
+    The object hierarchy is as follows.
 
     CoreArtist -> CoreAlbum -> CoreDisc -> CoreSong
 
@@ -73,79 +75,100 @@ class CoreModel(GObject.GObject):
 
     _recent_size = 21
 
-    def __init__(self, application):
+    def __init__(self, application: Application) -> None:
         """Initiate the CoreModel object
 
         :param Application application: The Application instance to use
         """
         super().__init__()
 
-        self._flatten_model = None
-        self._player_signal_id = None
-        self._current_playlist_model = None
-        self._previous_playlist_model = None
-
-        self._songs_model_proxy = Gio.ListStore.new(Gio.ListModel)
-        self._songs_model = Gfm.FlattenListModel.new(
-            CoreSong, self._songs_model_proxy)
-        self._songliststore = SongListStore(self._songs_model)
-
         self._application = application
+        self._flatten_model: Optional[Gtk.FlattenListModel] = None
+        self._player_signal_id = 0
+        self._current_playlist_model: Optional[Union[
+            Gtk.FlattenListModel, Gtk.SortListModel, Gio.ListModel]] = None
+        self._previous_playlist_model: Optional[Union[
+            Gtk.FlattenListModel, Gtk.SortListModel, Gio.ListModel]] = None
 
-        self._albums_model_proxy = Gio.ListStore.new(Gio.ListModel)
-        self._albums_model = Gfm.FlattenListModel.new(
-            CoreAlbum, self._albums_model_proxy)
-        self._albums_model_sort = Gfm.SortListModel.new(self._albums_model)
-        self._albums_model_sort.set_sort_func(
-            utils.wrap_list_store_sort_func(self._albums_sort))
+        self._songs_model_proxy: Gio.ListStore = Gio.ListStore.new(
+            Gio.ListModel)
+        self._flatten_songs_model = Gtk.FlattenListModel.new(
+            self._songs_model_proxy)
+        self._songs_model = Gtk.SortListModel.new(self._flatten_songs_model)
+        sorter = Gtk.CustomSorter()
+        sorter.set_sort_func(self._songs_sort)
+        self._songs_model.props.incremental = True
+        self._songs_model.set_sorter(sorter)
 
-        self._artists_model_proxy = Gio.ListStore.new(Gio.ListModel)
-        self._artists_model = Gfm.FlattenListModel.new(
-            CoreArtist, self._artists_model_proxy)
-        self._artists_model_sort = Gfm.SortListModel.new(self._artists_model)
-        self._artists_model_sort.set_sort_func(
-            utils.wrap_list_store_sort_func(self._artist_sort))
+        self._albums_model_proxy: Gio.ListStore = Gio.ListStore.new(
+            Gio.ListModel)
+        self._albums_model: Gtk.FlattenListModel = Gtk.FlattenListModel.new(
+            self._albums_model_proxy)
+        self._albums_model_sort: Gtk.SortListModel = Gtk.SortListModel.new(
+            self._albums_model)
+        albums_sorter = Gtk.CustomSorter()
+        albums_sorter.set_sort_func(self._albums_sort)
+        self._albums_model_sort.set_sorter(albums_sorter)
 
-        self._playlist_model = Gio.ListStore.new(CoreSong)
-        self._playlist_model_sort = Gfm.SortListModel.new(self._playlist_model)
-        self._playlist_model_recent = Gfm.SliceListModel.new(
-            self._playlist_model_sort, 0, self._recent_size)
-        self._active_core_object = None
+        self._artists_model_proxy: Gio.ListStore = Gio.ListStore.new(
+            Gio.ListModel)
+        self._artists_model: Gtk.FlattenListModel = Gtk.FlattenListModel.new(
+            self._artists_model_proxy)
+        self._artists_model_sort: Gtk.SortListModel = Gtk.SortListModel.new(
+            self._artists_model)
+        artists_sorter = Gtk.CustomSorter()
+        artists_sorter.set_sort_func(self._artist_sort)
+        self._artists_model_sort.set_sorter(artists_sorter)
 
-        self._songs_search_proxy = Gio.ListStore.new(Gfm.FilterListModel)
-        self._songs_search_flatten = Gfm.FlattenListModel.new(CoreSong)
+        self._playlist_model: Gio.ListStore = Gio.ListStore.new(CoreSong)
+        self._playlist_model_sort: Gtk.SortListModel = Gtk.SortListModel.new(
+            self._playlist_model)
+        self._playlist_model_recent: Gtk.SliceListModel = (
+            Gtk.SliceListModel.new(
+                self._playlist_model_sort, 0, self._recent_size))
+        self._active_core_object: Optional[Union[
+            CoreAlbum, CoreArtist, Playlist]] = None
+
+        self._songs_search_proxy: Gio.ListStore = Gio.ListStore.new(
+            Gtk.FilterListModel)
+        self._songs_search_flatten: Gtk.FlattenListModel = (
+            Gtk.FlattenListModel())
         self._songs_search_flatten.set_model(self._songs_search_proxy)
 
-        self._albums_search_model = Gfm.FilterListModel.new(
-            self._albums_model)
-        self._albums_search_model.set_filter_func(lambda a: False)
+        self._albums_search_proxy: Gio.Liststore = Gio.ListStore.new(
+            Gtk.FilterListModel)
+        self._albums_search_flatten: Gtk.FlattenListModel = (
+            Gtk.FlattenListModel())
+        self._albums_search_flatten.set_model(self._albums_search_proxy)
+        self._albums_search_filter: Gtk.FilterListModel = (
+            Gtk.FilterListModel.new(Gtk.AnyFilter()))
 
-        self._albums_search_filter = Gfm.FilterListModel.new(
-            self._albums_search_model)
+        self._artists_search_proxy: Gio.Liststore = Gio.ListStore.new(
+            Gtk.FilterListModel)
+        self._artists_search_flatten: Gtk.FlattenListModel = (
+            Gtk.FlattenListModel())
+        self._artists_search_flatten.set_model(self._artists_search_proxy)
+        self._artists_search_filter: Gtk.FilterListModel = (
+            Gtk.FilterListModel.new(Gtk.AnyFilter()))
 
-        self._artists_search_model = Gfm.FilterListModel.new(
-            self._artists_model)
-        self._artists_search_model.set_filter_func(lambda a: False)
-
-        self._artists_search_filter = Gfm.FilterListModel.new(
-            self._artists_search_model)
-
-        self._playlists_model = Gio.ListStore.new(Playlist)
-        self._playlists_model_filter = Gfm.FilterListModel.new(
-            self._playlists_model)
-        self._playlists_model_sort = Gfm.SortListModel.new(
+        self._playlists_model: Gio.ListStore = Gio.ListStore.new(Playlist)
+        self._playlists_model_filter: Gtk.FilterListModel = (
+            Gtk.FilterListModel.new(self._playlists_model))
+        self._playlists_model_sort: Gtk.SortListModel = Gtk.SortListModel.new(
             self._playlists_model_filter)
-        self._playlists_model_sort.set_sort_func(
-            utils.wrap_list_store_sort_func(self._playlists_sort))
+        playlists_sorter = Gtk.CustomSorter()
+        playlists_sorter.set_sort_func(self._playlists_sort)
+        self._playlists_model_sort.set_sorter(playlists_sorter)
 
-        self._user_playlists_model_filter = Gfm.FilterListModel.new(
-            self._playlists_model)
-        self._user_playlists_model_sort = Gfm.SortListModel.new(
-            self._user_playlists_model_filter)
-        self._user_playlists_model_sort.set_sort_func(
-            utils.wrap_list_store_sort_func(self._playlists_sort))
+        self._user_playlists_model_filter: Gtk.FilterListModel = (
+            Gtk.FilterListModel.new(self._playlists_model))
+        self._user_playlists_model_sort: Gtk.SortListModel = (
+            Gtk.SortListModel.new(self._user_playlists_model_filter))
+        user_playlists_sorter = Gtk.CustomSorter()
+        user_playlists_sorter.set_sort_func(self._playlists_sort)
+        self._user_playlists_model_sort.set_sorter(user_playlists_sorter)
 
-        self._search = application.props.search
+        self._search: Search = application.props.search
 
         self._songs_model.connect(
             "items-changed", self._on_songs_items_changed)
@@ -162,37 +185,50 @@ class CoreModel(GObject.GObject):
         else:
             self.props.songs_available = False
 
-    def _filter_selected(self, coresong):
-        return coresong.props.selected
+    def _songs_sort(
+            self, song_a: CoreSong, song_b: CoreSong, data: Any = None) -> int:
+        title_a = song_a.props.title
+        title_b = song_b.props.title
+        song_cmp = (utils.normalize_caseless(title_a)
+                    == utils.normalize_caseless(title_b))
+        if not song_cmp:
+            return utils.natural_sort_names(title_a, title_b)
 
-    def _albums_sort(self, album_a, album_b):
+        artist_a = song_a.props.artist
+        artist_b = song_b.props.artist
+        artist_cmp = (utils.normalize_caseless(artist_a)
+                      == utils.normalize_caseless(artist_b))
+        if not artist_cmp:
+            return utils.natural_sort_names(artist_a, artist_b)
+
+        return utils.natural_sort_names(song_a.props.album, song_b.props.album)
+
+    def _albums_sort(self, album_a, album_b, data=None):
         return utils.natural_sort_names(
             album_a.props.title, album_b.props.title)
 
-    def _artist_sort(self, artist_a, artist_b):
+    def _artist_sort(self, artist_a, artist_b, data=None):
         return utils.natural_sort_names(
             artist_a.props.artist, artist_b.props.artist)
 
-    def _playlists_sort(self, playlist_a, playlist_b):
+    def _playlists_sort(self, playlist_a, playlist_b, data=None):
         if playlist_a.props.is_smart:
             if not playlist_b.props.is_smart:
-                return -1
+                return Gtk.Ordering.SMALLER
             return utils.natural_sort_names(
                 playlist_a.props.title, playlist_b.props.title)
 
         if playlist_b.props.is_smart:
-            return 1
+            return Gtk.Ordering.LARGER
 
-        if (playlist_a.props.creation_date is None
-                or playlist_b.props.creation_date is None):
-            return 0
-
-        # cannot use GLib.DateTime.compare
-        # https://gitlab.gnome.org/GNOME/pygobject/issues/334
-        # newest first
-        date_diff = playlist_b.props.creation_date.difference(
-            playlist_a.props.creation_date)
-        return math.copysign(1, date_diff)
+        date_compare = GLib.DateTime.compare(
+            playlist_b.props.creation_date, playlist_a.props.creation_date)
+        if date_compare == -1:
+            return Gtk.Ordering.SMALLER
+        elif date_compare == 1:
+            return Gtk.Ordering.LARGER
+        else:
+            return Gtk.Ordering.EQUAL
 
     def _set_player_model(self, playlist_type, model):
         """Set the model for PlayerPlaylist to use
@@ -234,9 +270,9 @@ class CoreModel(GObject.GObject):
             if song.props.state in played_states:
                 song.props.state = SongWidget.State.UNPLAYED
 
-        if self._player_signal_id is not None:
+        if self._player_signal_id != 0:
             self._current_playlist_model.disconnect(self._player_signal_id)
-            self._player_signal_id = None
+            self._player_signal_id = 0
             self._current_playlist_model = None
 
         songs_added = []
@@ -247,8 +283,7 @@ class CoreModel(GObject.GObject):
             for disc in model:
                 proxy_model.append(disc.props.model)
 
-            self._flatten_model = Gfm.FlattenListModel.new(
-                CoreSong, proxy_model)
+            self._flatten_model = Gtk.FlattenListModel.new(proxy_model)
             self._current_playlist_model = self._flatten_model
 
             for model_song in self._flatten_model:
@@ -263,8 +298,7 @@ class CoreModel(GObject.GObject):
                 for disc in artist_album.model:
                     proxy_model.append(disc.props.model)
 
-            self._flatten_model = Gfm.FlattenListModel.new(
-                CoreSong, proxy_model)
+            self._flatten_model = Gtk.FlattenListModel.new(proxy_model)
             self._current_playlist_model = self._flatten_model
 
             for model_song in self._flatten_model:
@@ -273,9 +307,9 @@ class CoreModel(GObject.GObject):
                 songs_added.append(song)
 
         elif playlist_type == PlayerPlaylist.Type.SONGS:
-            self._current_playlist_model = self._songliststore.props.model
+            self._current_playlist_model = self._songs_model
 
-            for song in self._songliststore.props.model:
+            for song in self._songs_model:
                 songs_added.append(song)
 
                 if song.props.state == SongWidget.State.PLAYING:
@@ -315,7 +349,7 @@ class CoreModel(GObject.GObject):
         """
         return self._active_core_object
 
-    @active_core_object.setter
+    @active_core_object.setter  # type: ignore
     def active_core_object(self, value):
         """Set the current playing core object
         (album, artist, playlist, search result or song).
@@ -350,7 +384,7 @@ class CoreModel(GObject.GObject):
         return self._songs_model
 
     @GObject.Property(
-        type=Gfm.FlattenListModel, default=None,
+        type=Gtk.FlattenListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def songs_proxy(self):
         return self._songs_model_proxy
@@ -361,7 +395,7 @@ class CoreModel(GObject.GObject):
         return self._albums_model
 
     @GObject.Property(
-        type=Gfm.FlattenListModel, default=None,
+        type=Gtk.FlattenListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def albums_proxy(self):
         return self._albums_model_proxy
@@ -372,7 +406,7 @@ class CoreModel(GObject.GObject):
         return self._artists_model
 
     @GObject.Property(
-        type=Gfm.FlattenListModel, default=None,
+        type=Gtk.FlattenListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def artists_proxy(self):
         return self._artists_model_proxy
@@ -383,25 +417,25 @@ class CoreModel(GObject.GObject):
         return self._playlist_model
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def albums_sort(self):
         return self._albums_model_sort
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def artists_sort(self):
         return self._artists_model_sort
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def playlist_sort(self):
         return self._playlist_model_sort
 
     @GObject.Property(
-        type=Gfm.SliceListModel, default=None,
+        type=Gtk.SliceListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def recent_playlist(self):
         return self._playlist_model_recent
@@ -413,7 +447,7 @@ class CoreModel(GObject.GObject):
         return self._recent_size // 2
 
     @GObject.Property(
-        type=Gfm.FilterListModel, default=None,
+        type=Gtk.FilterListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def songs_search(self):
         return self._songs_search_flatten
@@ -425,33 +459,38 @@ class CoreModel(GObject.GObject):
         return self._songs_search_proxy
 
     @GObject.Property(
-        type=Gfm.FilterListModel, default=None,
+        type=Gtk.FlattenListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
-    def albums_search(self):
-        return self._albums_search_model
+    def albums_search(self) -> Gtk.FlattenListModel:
+        return self._albums_search_flatten
 
     @GObject.Property(
-        type=Gfm.FilterListModel, default=None,
+        type=Gtk.FilterListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def albums_search_filter(self):
         return self._albums_search_filter
 
     @GObject.Property(
-        type=Gfm.FilterListModel, default=None,
-        flags=GObject.ParamFlags.READABLE)
-    def artists_search(self):
-        return self._artists_search_model
+        type=Gio.ListStore, default=None, flags=GObject.ParamFlags.READABLE)
+    def albums_search_proxy(self) -> Gio.ListStore:
+        return self._albums_search_proxy
 
     @GObject.Property(
-        type=Gfm.FilterListModel, default=None,
+        type=Gtk.FlattenListModel, default=None,
+        flags=GObject.ParamFlags.READABLE)
+    def artists_search(self) -> Gtk.FlattenListModel:
+        return self._artists_search_flatten
+
+    @GObject.Property(
+        type=Gtk.FilterListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def artists_search_filter(self):
         return self._artists_search_filter
 
     @GObject.Property(
-        type=Gtk.ListStore, default=None, flags=GObject.ParamFlags.READABLE)
-    def songs_gtkliststore(self):
-        return self._songliststore
+        type=Gio.ListStore, default=None, flags=GObject.ParamFlags.READABLE)
+    def artists_search_proxy(self) -> Gio.ListStore:
+        return self._artists_search_proxy
 
     @GObject.Property(
         type=Gio.ListStore, default=None, flags=GObject.ParamFlags.READABLE)
@@ -459,25 +498,25 @@ class CoreModel(GObject.GObject):
         return self._playlists_model
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def playlists_sort(self):
         return self._playlists_model_sort
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def playlists_filter(self):
         return self._playlists_model_filter
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def user_playlists_sort(self):
         return self._user_playlists_model_sort
 
     @GObject.Property(
-        type=Gfm.SortListModel, default=None,
+        type=Gtk.SortListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def user_playlists_filter(self):
         return self._user_playlists_model_filter
