@@ -25,9 +25,8 @@
 from gi.repository import Gdk, GObject, Gtk
 
 from gnomemusic.coreartist import CoreArtist
-from gnomemusic.defaulticon import DefaultIcon
-from gnomemusic.utils import ArtSize
-from gnomemusic.widgets.artstack import ArtStack  # noqa: F401
+from gnomemusic.coverpaintable import CoverPaintable
+from gnomemusic.utils import ArtSize, DefaultIconType
 from gnomemusic.widgets.twolinetip import TwoLineTip
 
 
@@ -41,9 +40,8 @@ class ArtistSearchTile(Gtk.FlowBoxChild):
     __gtype_name__ = "ArtistSearchTile"
 
     _artist_label = Gtk.Template.Child()
-    _art_stack = Gtk.Template.Child()
+    _cover_image = Gtk.Template.Child()
     _check = Gtk.Template.Child()
-    _events = Gtk.Template.Child()
 
     coreartist = GObject.Property(
         type=CoreArtist, default=None, flags=GObject.ParamFlags.READWRITE)
@@ -61,9 +59,12 @@ class ArtistSearchTile(Gtk.FlowBoxChild):
 
         self.props.coreartist = coreartist
 
-        self._art_stack.props.size = ArtSize.MEDIUM
-        self._art_stack.props.art_type = DefaultIcon.Type.ARTIST
-        self._art_stack.props.coreobject = self.props.coreartist
+        self._cover_image.set_size_request(
+            ArtSize.MEDIUM.width, ArtSize.MEDIUM.height)
+        self._cover_image.props.paintable = CoverPaintable(
+            self, ArtSize.MEDIUM, DefaultIconType.ARTIST)
+
+        self._cover_image.props.paintable.props.coreobject = coreartist
 
         self._tooltip = TwoLineTip()
         self._tooltip.props.subtitle_visible = False
@@ -84,14 +85,11 @@ class ArtistSearchTile(Gtk.FlowBoxChild):
             "selection-mode", self._check, "visible",
             GObject.BindingFlags.BIDIRECTIONAL)
 
-        self._events.add_events(Gdk.EventMask.TOUCH_MASK)
-
-        self.show()
-
     @Gtk.Template.Callback()
-    def _on_artist_event(self, evbox, event, data=None):
+    def _on_artist_event(self, gesture_click, n_press, x, y):
+        state = gesture_click.get_current_event_state()
         modifiers = Gtk.accelerator_get_default_mod_mask()
-        if ((event.get_state() & modifiers) == Gdk.ModifierType.CONTROL_MASK
+        if (state & modifiers == Gdk.ModifierType.CONTROL_MASK
                 and not self.props.selection_mode):
             self.props.selection_mode = True
 
