@@ -25,9 +25,9 @@
 from gettext import gettext as _
 from gi.repository import Gio, GLib, GObject, Gtk
 
-from gnomemusic.defaulticon import DefaultIcon
+from gnomemusic.coverpaintable import CoverPaintable
 from gnomemusic.gstplayer import Playback
-from gnomemusic.utils import ArtSize
+from gnomemusic.utils import ArtSize, DefaultIconType
 from gnomemusic.player import Player, RepeatMode
 from gnomemusic.widgets.smoothscale import SmoothScale  # noqa: F401
 from gnomemusic.widgets.twolinetip import TwoLineTip
@@ -44,13 +44,11 @@ class PlayerToolbar(Gtk.ActionBar):
     __gtype_name__ = 'PlayerToolbar'
 
     _artist_label = Gtk.Template.Child()
-    _art_stack = Gtk.Template.Child()
-    _buttons_and_scale = Gtk.Template.Child()
+    _cover_image = Gtk.Template.Child()
     _duration_label = Gtk.Template.Child()
     _next_button = Gtk.Template.Child()
-    _pause_image = Gtk.Template.Child()
     _play_button = Gtk.Template.Child()
-    _play_image = Gtk.Template.Child()
+    _play_pause_image = Gtk.Template.Child()
     _prev_button = Gtk.Template.Child()
     _progress_scale = Gtk.Template.Child()
     _progress_time_label = Gtk.Template.Child()
@@ -64,18 +62,13 @@ class PlayerToolbar(Gtk.ActionBar):
 
         self._player = None
 
-        self._art_stack.props.size = ArtSize.SMALL
-        self._art_stack.props.art_type = DefaultIcon.Type.ALBUM
+        self._cover_image.set_size_request(
+            ArtSize.SMALL.width, ArtSize.SMALL.height)
+        self._cover_image.props.paintable = CoverPaintable(
+            self, ArtSize.SMALL, DefaultIconType.ALBUM)
 
         self._tooltip = TwoLineTip()
 
-        # A centered widget has an expand child property set to False
-        # by default. It needs to be True to have a progress scale
-        # at the correct size.
-        main_container = self._buttons_and_scale.get_parent()
-        main_container.child_set_property(
-            self._buttons_and_scale, "expand", True)
-
         repeat_menu = Gio.Menu.new()
         for mode in RepeatMode:
             item = Gio.MenuItem.new()
@@ -88,6 +81,10 @@ class PlayerToolbar(Gtk.ActionBar):
         self._repeat_action: Gio.SimpleAction = Gio.SimpleAction.new_stateful(
             "repeat", GLib.VariantType.new("s"), GLib.Variant("s", ""))
 
+        action_group = Gio.SimpleActionGroup()
+        action_group.add_action(self._repeat_action)
+        self.insert_action_group("playertoolbar", action_group)
+
     # FIXME: This is a workaround for not being able to pass the player
     # object via init when using Gtk.Builder.
     @GObject.Property(type=Player, default=None)
@@ -122,9 +119,6 @@ class PlayerToolbar(Gtk.ActionBar):
         self._repeat_action.set_state(
             GLib.Variant("s", str(repeat_mode.value)))
         self._repeat_action.connect("activate", self._repeat_menu_changed)
-        action_group = Gio.SimpleActionGroup()
-        action_group.add_action(self._repeat_action)
-        self.insert_action_group("playertoolbar", action_group)
 
         self._sync_repeat_image()
 
@@ -133,7 +127,6 @@ class PlayerToolbar(Gtk.ActionBar):
         self._repeat_action.set_state(new_state)
         new_mode = new_state.get_string()
         self._player.props.repeat_mode = RepeatMode(int(new_mode))
-        self._repeat_menu_button.props.active = False
 
     @Gtk.Template.Callback()
     def _on_progress_value_changed(self, progress_scale):
@@ -158,26 +151,26 @@ class PlayerToolbar(Gtk.ActionBar):
 
     def _sync_repeat_image(self) -> None:
         self._repeat_image.set_from_icon_name(
-            self._player.props.repeat_mode.icon, Gtk.IconSize.MENU)
+            self._player.props.repeat_mode.icon)
 
     def _sync_playing(self, player, state):
         if (self._player.props.state == Playback.STOPPED
                 and not self._player.props.has_next
                 and not self._player.props.has_previous):
-            self.hide()
+            self.props.revealed = False
             return
 
-        self.show()
+        self.props.revealed = True
 
         if self._player.props.state == Playback.PLAYING:
-            image = self._pause_image
+            icon_name = "media-playback-pause-symbolic"
             tooltip = _("Pause")
         else:
-            image = self._play_image
+            icon_name = "media-playback-start-symbolic"
             tooltip = _("Play")
 
-        if self._play_button.get_image() != image:
-            self._play_button.set_image(image)
+        if self._play_pause_image.props.icon_name != icon_name:
+            self._play_pause_image.props.icon_name = icon_name
 
         self._play_button.set_tooltip_text(tooltip)
 
@@ -207,7 +200,7 @@ class PlayerToolbar(Gtk.ActionBar):
         self._tooltip.props.title = title
         self._tooltip.props.subtitle = artist
 
-        self._art_stack.props.coreobject = coresong
+        self._cover_image.props.paintable.props.coreobject = coresong
 
     @Gtk.Template.Callback()
     def _on_tooltip_query(self, widget, x, y, kb, tooltip, data=None):
