@@ -36,7 +36,7 @@ class StarImage(Gtk.Image):
         self._hover = False
 
         self.get_style_context().add_class("star")
-        self.show_all()
+        self.props.visible = True
 
     @GObject.Property(type=bool, default=False)
     def favorite(self):
@@ -47,7 +47,7 @@ class StarImage(Gtk.Image):
         """
         return self._favorite
 
-    @favorite.setter
+    @favorite.setter  # type: ignore
     def favorite(self, value):
         """Set favorite
 
@@ -66,7 +66,7 @@ class StarImage(Gtk.Image):
     def hover(self):
         return self._hover
 
-    @hover.setter
+    @hover.setter  # type: ignore
     def hover(self, value):
         if value:
             self.set_state_flags(Gtk.StateFlags.PRELIGHT, False)
